"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const base_command_1 = require("./base-command");
const app_1 = require("./app");
const locale_1 = require("@joplin/lib/locale");
const BaseModel_1 = require("@joplin/lib/BaseModel");
const Note_1 = require("@joplin/lib/models/Note");
class Command extends base_command_1.default {
    usage() {
        return 'cp <note> [notebook]';
    }
    description() {
        return (0, locale_1._)('Duplicates the notes matching <note> to [notebook]. If no notebook is specified the note is duplicated in the current notebook.');
    }
    action(args) {
        return __awaiter(this, void 0, void 0, function* () {
            let folder = null;
            if (args['notebook']) {
                folder = yield (0, app_1.default)().loadItem(BaseModel_1.default.TYPE_FOLDER, args['notebook']);
            }
            else {
                folder = (0, app_1.default)().currentFolder();
            }
            if (!folder)
                throw new Error((0, locale_1._)('Cannot find "%s".', args['notebook']));
            const notes = yield (0, app_1.default)().loadItems(BaseModel_1.default.TYPE_NOTE, args['note']);
            if (!notes.length)
                throw new Error((0, locale_1._)('Cannot find "%s".', args['note']));
            for (let i = 0; i < notes.length; i++) {
                const newNote = yield Note_1.default.copyToFolder(notes[i].id, folder.id);
                void Note_1.default.updateGeolocation(newNote.id);
            }
        });
    }
}
module.exports = Command;
//# sourceMappingURL=command-cp.js.map