"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const base_command_1 = require("./base-command");
const app_1 = require("./app");
const locale_1 = require("@joplin/lib/locale");
const BaseModel_1 = require("@joplin/lib/BaseModel");
const Note_1 = require("@joplin/lib/models/Note");
const time_1 = require("@joplin/lib/time");
class Command extends base_command_1.default {
    usage() {
        return 'done <note>';
    }
    description() {
        return (0, locale_1._)('Marks a to-do as done.');
    }
    static handleAction(commandInstance, args, isCompleted) {
        return __awaiter(this, void 0, void 0, function* () {
            const note = yield (0, app_1.default)().loadItem(BaseModel_1.default.TYPE_NOTE, args.note);
            commandInstance.encryptionCheck(note);
            if (!note)
                throw new Error((0, locale_1._)('Cannot find "%s".', args.note));
            if (!note.is_todo)
                throw new Error((0, locale_1._)('Note is not a to-do: "%s"', args.note));
            const todoCompleted = !!note.todo_completed;
            if (isCompleted === todoCompleted)
                return;
            yield Note_1.default.save({
                id: note.id,
                todo_completed: isCompleted ? time_1.default.unixMs() : 0,
            });
        });
    }
    action(args) {
        return __awaiter(this, void 0, void 0, function* () {
            yield Command.handleAction(this, args, true);
        });
    }
}
module.exports = Command;
//# sourceMappingURL=command-done.js.map