"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const fs = require("fs-extra");
const base_command_1 = require("./base-command");
const utils_1 = require("@joplin/utils");
const uuid_1 = require("@joplin/lib/uuid");
const app_1 = require("./app");
const locale_1 = require("@joplin/lib/locale");
const Note_1 = require("@joplin/lib/models/Note");
const Setting_1 = require("@joplin/lib/models/Setting");
const BaseModel_1 = require("@joplin/lib/BaseModel");
class Command extends base_command_1.default {
    usage() {
        return 'edit <note>';
    }
    description() {
        return (0, locale_1._)('Edit note.');
    }
    action(args) {
        return __awaiter(this, void 0, void 0, function* () {
            let tempFilePath = null;
            const onFinishedEditing = () => __awaiter(this, void 0, void 0, function* () {
                if (tempFilePath)
                    fs.removeSync(tempFilePath);
            });
            const textEditorPath = () => {
                if (Setting_1.default.value('editor'))
                    return Setting_1.default.value('editor');
                if (process.env.EDITOR)
                    return process.env.EDITOR;
                throw new Error((0, locale_1._)('No text editor is defined. Please set it using `config editor <editor-path>`'));
            };
            try {
                // -------------------------------------------------------------------------
                // Load note or create it if it doesn't exist
                // -------------------------------------------------------------------------
                const title = args['note'];
                if (!(0, app_1.default)().currentFolder())
                    throw new Error((0, locale_1._)('No active notebook.'));
                let note = yield (0, app_1.default)().loadItem(BaseModel_1.default.TYPE_NOTE, title);
                this.encryptionCheck(note);
                if (!note) {
                    const ok = yield this.prompt((0, locale_1._)('Note does not exist: "%s". Create it?', title));
                    if (!ok)
                        return;
                    note = yield Note_1.default.save({ title: title, parent_id: (0, app_1.default)().currentFolder().id });
                    note = yield Note_1.default.load(note.id);
                }
                // -------------------------------------------------------------------------
                // Create the file to be edited and prepare the editor program arguments
                // -------------------------------------------------------------------------
                let editorPath = textEditorPath();
                let editorArgs = (0, utils_1.splitCommandString)(editorPath);
                editorPath = editorArgs[0];
                editorArgs = editorArgs.splice(1);
                const originalContent = yield Note_1.default.serializeForEdit(note);
                tempFilePath = `${Setting_1.default.value('tempDir')}/${uuid_1.default.create()}.md`;
                editorArgs.push(tempFilePath);
                yield fs.writeFile(tempFilePath, originalContent);
                // -------------------------------------------------------------------------
                // Start editing the file
                // -------------------------------------------------------------------------
                this.logger().info('Disabling fullscreen...');
                (0, app_1.default)().gui().showModalOverlay((0, locale_1._)('Starting to edit note. Close the editor to get back to the prompt.'));
                yield (0, app_1.default)().gui().forceRender();
                const termState = (0, app_1.default)().gui().termSaveState();
                const spawnSync = require('child_process').spawnSync;
                const result = spawnSync(editorPath, editorArgs, { stdio: 'inherit' });
                if (result.error)
                    this.stdout((0, locale_1._)('Error opening note in editor: %s', result.error.message));
                (0, app_1.default)().gui().termRestoreState(termState);
                (0, app_1.default)().gui().hideModalOverlay();
                (0, app_1.default)().gui().forceRender();
                // -------------------------------------------------------------------------
                // Save the note and clean up
                // -------------------------------------------------------------------------
                const updatedContent = yield fs.readFile(tempFilePath, 'utf8');
                if (updatedContent !== originalContent) {
                    const updatedNote = yield Note_1.default.unserializeForEdit(updatedContent);
                    updatedNote.id = note.id;
                    yield Note_1.default.save(updatedNote);
                    this.stdout((0, locale_1._)('Note has been saved.'));
                }
                this.dispatch({
                    type: 'NOTE_SELECT',
                    id: note.id,
                });
                yield onFinishedEditing();
            }
            catch (error) {
                yield onFinishedEditing();
                throw error;
            }
        });
    }
}
module.exports = Command;
//# sourceMappingURL=command-edit.js.map