"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const BaseCommand = require('./base-command').default;
const app_1 = require("./app");
const locale_1 = require("@joplin/lib/locale");
const BaseModel_1 = require("@joplin/lib/BaseModel");
const Folder_1 = require("@joplin/lib/models/Folder");
class Command extends BaseCommand {
    usage() {
        return 'mkbook <new-notebook>';
    }
    description() {
        return (0, locale_1._)('Creates a new notebook.');
    }
    options() {
        return [
            ['-p, --parent <parent-notebook>', (0, locale_1._)('Create a new notebook under a parent notebook.')],
        ];
    }
    // validDestinationFolder check for presents and ambiguous folders
    validDestinationFolder(targetFolder) {
        return __awaiter(this, void 0, void 0, function* () {
            const destinationFolder = yield (0, app_1.default)().loadItem(BaseModel_1.default.TYPE_FOLDER, targetFolder);
            if (!destinationFolder) {
                throw new Error((0, locale_1._)('Cannot find: "%s"', targetFolder));
            }
            const destinationDups = yield Folder_1.default.search({ titlePattern: targetFolder, limit: 2 });
            if (destinationDups.length > 1) {
                throw new Error((0, locale_1._)('Ambiguous notebook "%s". Please use short notebook id instead - press "ti" to see the short notebook id', targetFolder));
            }
            return destinationFolder;
        });
    }
    saveAndSwitchFolder(newFolder) {
        return __awaiter(this, void 0, void 0, function* () {
            const folder = yield Folder_1.default.save(newFolder, { userSideValidation: true });
            (0, app_1.default)().switchCurrentFolder(folder);
        });
    }
    action(args) {
        return __awaiter(this, void 0, void 0, function* () {
            const targetFolder = args.options.parent;
            const newFolder = {
                title: args['new-notebook'],
            };
            if (targetFolder) {
                const destinationFolder = yield this.validDestinationFolder(targetFolder);
                newFolder.parent_id = destinationFolder.id;
                yield this.saveAndSwitchFolder(newFolder);
            }
            else {
                yield this.saveAndSwitchFolder(newFolder);
            }
        });
    }
}
module.exports = Command;
//# sourceMappingURL=command-mkbook.js.map