"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const base_command_1 = require("./base-command");
const app_1 = require("./app");
const locale_1 = require("@joplin/lib/locale");
const BaseModel_1 = require("@joplin/lib/BaseModel");
const Folder_1 = require("@joplin/lib/models/Folder");
const Note_1 = require("@joplin/lib/models/Note");
class Command extends base_command_1.default {
    usage() {
        return 'ren <item> <name>';
    }
    description() {
        return (0, locale_1._)('Renames the given <item> (note or notebook) to <name>.');
    }
    action(args) {
        return __awaiter(this, void 0, void 0, function* () {
            const pattern = args['item'];
            const name = args['name'];
            const item = yield (0, app_1.default)().loadItem('folderOrNote', pattern);
            this.encryptionCheck(item);
            if (!item)
                throw new Error((0, locale_1._)('Cannot find "%s".', pattern));
            const newItem = {
                id: item.id,
                title: name,
                type_: item.type_,
            };
            if (item.type_ === BaseModel_1.default.TYPE_FOLDER) {
                yield Folder_1.default.save(newItem);
            }
            else {
                yield Note_1.default.save(newItem);
            }
        });
    }
}
module.exports = Command;
//# sourceMappingURL=command-ren.js.map