"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const base_command_1 = require("./base-command");
const app_1 = require("./app");
const locale_1 = require("@joplin/lib/locale");
const Folder_1 = require("@joplin/lib/models/Folder");
const BaseModel_1 = require("@joplin/lib/BaseModel");
class Command extends base_command_1.default {
    usage() {
        return 'rmbook <notebook>';
    }
    description() {
        return (0, locale_1._)('Deletes the given notebook.');
    }
    options() {
        return [['-f, --force', (0, locale_1._)('Deletes the notebook without asking for confirmation.')]];
    }
    action(args) {
        return __awaiter(this, void 0, void 0, function* () {
            const pattern = args['notebook'];
            const force = args.options && args.options.force === true;
            const folder = yield (0, app_1.default)().loadItem(BaseModel_1.default.TYPE_FOLDER, pattern);
            if (!folder)
                throw new Error((0, locale_1._)('Cannot find "%s".', pattern));
            const ok = force ? true : yield this.prompt((0, locale_1._)('Delete notebook? All notes and sub-notebooks within this notebook will also be deleted.'), { booleanAnswerDefault: 'n' });
            if (!ok)
                return;
            yield Folder_1.default.delete(folder.id);
        });
    }
}
module.exports = Command;
//# sourceMappingURL=command-rmbook.js.map