"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const base_command_1 = require("./base-command");
const app_1 = require("./app");
const locale_1 = require("@joplin/lib/locale");
const Note_1 = require("@joplin/lib/models/Note");
const BaseModel_1 = require("@joplin/lib/BaseModel");
class Command extends base_command_1.default {
    usage() {
        return 'rmnote <note-pattern>';
    }
    description() {
        return (0, locale_1._)('Deletes the notes matching <note-pattern>.');
    }
    options() {
        return [['-f, --force', (0, locale_1._)('Deletes the notes without asking for confirmation.')]];
    }
    action(args) {
        return __awaiter(this, void 0, void 0, function* () {
            const pattern = args['note-pattern'];
            const force = args.options && args.options.force === true;
            const notes = yield (0, app_1.default)().loadItems(BaseModel_1.default.TYPE_NOTE, pattern);
            if (!notes.length)
                throw new Error((0, locale_1._)('Cannot find "%s".', pattern));
            const ok = force ? true : yield this.prompt(notes.length > 1 ? (0, locale_1._)('%d notes match this pattern. Delete them?', notes.length) : (0, locale_1._)('Delete note?'), { booleanAnswerDefault: 'n' });
            if (!ok)
                return;
            const ids = notes.map((n) => n.id);
            yield Note_1.default.batchDelete(ids);
        });
    }
}
module.exports = Command;
//# sourceMappingURL=command-rmnote.js.map