"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const Setting_1 = require("@joplin/lib/models/Setting");
const types_1 = require("@joplin/lib/services/plugins/api/types");
const shim_1 = require("@joplin/lib/shim");
const BaseCommand = require('./base-command').default;
function settingTypeToSchemaType(type) {
    const map = {
        [types_1.SettingItemType.Int]: 'integer',
        [types_1.SettingItemType.String]: 'string',
        [types_1.SettingItemType.Bool]: 'boolean',
        [types_1.SettingItemType.Array]: 'array',
        [types_1.SettingItemType.Object]: 'object',
        [types_1.SettingItemType.Button]: '',
    };
    const r = map[type];
    if (r === '')
        return '';
    if (!r)
        throw new Error(`Unsupported type: ${type}`);
    return r;
}
class Command extends BaseCommand {
    usage() {
        return 'settingschema <file>';
    }
    description() {
        return 'Build the setting schema file';
    }
    enabled() {
        return false;
    }
    action(args) {
        return __awaiter(this, void 0, void 0, function* () {
            const schema = {
                title: 'JSON schema for Joplin setting files',
                '$id': Setting_1.default.schemaUrl,
                '$schema': 'https://json-schema.org/draft-07/schema#',
                type: 'object',
                properties: {},
            };
            const metadata = Setting_1.default.metadata();
            for (const key of Object.keys(metadata)) {
                const md = metadata[key];
                const type = settingTypeToSchemaType(md.type);
                if (!type)
                    continue;
                const props = {};
                props.type = type;
                props.default = md.value;
                const description = [];
                if (md.label && md.label())
                    description.push(md.label());
                if (md.description && md.description('desktop'))
                    description.push(md.description('desktop'));
                if (description.length)
                    props.description = description.join('. ');
                if (md.isEnum)
                    props.enum = Object.keys(md.options()).map((v) => Setting_1.default.formatValue(key, v));
                if ('minimum' in md)
                    props.minimum = md.minimum;
                if ('maximum' in md)
                    props.maximum = md.maximum;
                if (!md.public || md.storage !== Setting_1.SettingStorage.File)
                    props['$comment'] = 'private';
                schema.properties[key] = props;
            }
            const outFilePath = args['file'];
            yield shim_1.default.fsDriver().writeFile(outFilePath, JSON.stringify(schema, null, '\t'), 'utf8');
        });
    }
}
module.exports = Command;
//# sourceMappingURL=command-settingschema.js.map