"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const BaseCommand = require('./base-command').default;
const registry_1 = require("@joplin/lib/registry");
const Note_1 = require("@joplin/lib/models/Note");
const uuid_1 = require("@joplin/lib/uuid");
const populateDatabase_1 = require("@joplin/lib/services/debug/populateDatabase");
const credentialFiles_1 = require("@joplin/lib/utils/credentialFiles");
const JoplinServerApi_1 = require("@joplin/lib/JoplinServerApi");
function randomElement(array) {
    if (!array.length)
        return null;
    return array[Math.floor(Math.random() * array.length)];
}
function itemCount(args) {
    const count = Number(args.arg0);
    if (!count || isNaN(count))
        throw new Error('Note count must be specified');
    return count;
}
class Command extends BaseCommand {
    usage() {
        return 'testing <command> [arg0]';
    }
    description() {
        return 'testing';
    }
    enabled() {
        return false;
    }
    options() {
        return [
            ['--folder-count <count>', 'Folders to create'],
            ['--note-count <count>', 'Notes to create'],
            ['--tag-count <count>', 'Tags to create'],
            ['--tags-per-note <count>', 'Tags per note'],
            ['--silent', 'Silent'],
        ];
    }
    action(args) {
        return __awaiter(this, void 0, void 0, function* () {
            const { command, options } = args;
            if (command === 'populate') {
                yield (0, populateDatabase_1.default)(registry_1.reg.db(), {
                    folderCount: options['folder-count'],
                    noteCount: options['note-count'],
                    tagCount: options['tag-count'],
                    tagsPerNote: options['tags-per-note'],
                    silent: options['silent'],
                });
            }
            const promises = [];
            if (command === 'createRandomNotes') {
                const noteCount = itemCount(args);
                for (let i = 0; i < noteCount; i++) {
                    promises.push(Note_1.default.save({
                        title: `Note ${uuid_1.default.createNano()}`,
                    }));
                }
            }
            if (command === 'updateRandomNotes') {
                const noteCount = itemCount(args);
                const noteIds = yield Note_1.default.allIds();
                for (let i = 0; i < noteCount; i++) {
                    const noteId = randomElement(noteIds);
                    promises.push(Note_1.default.save({
                        id: noteId,
                        title: `Note ${uuid_1.default.createNano()}`,
                    }));
                }
            }
            if (command === 'deleteRandomNotes') {
                const noteCount = itemCount(args);
                const noteIds = yield Note_1.default.allIds();
                for (let i = 0; i < noteCount; i++) {
                    const noteId = randomElement(noteIds);
                    promises.push(Note_1.default.delete(noteId));
                }
            }
            if (command === 'joplinServerParallelItemUpdate') {
                const randomContent = () => {
                    const charCount = Math.random() * 1000;
                    return 'a'.repeat(charCount);
                };
                const joplinServerAuth = JSON.parse(yield (0, credentialFiles_1.readCredentialFile)('joplin-server-test.json'));
                const api = new JoplinServerApi_1.default({
                    baseUrl: () => joplinServerAuth.baseUrl,
                    userContentBaseUrl: () => joplinServerAuth.userContentBaseUrl,
                    username: () => joplinServerAuth.email,
                    password: () => joplinServerAuth.password,
                });
                const apiPut = () => __awaiter(this, void 0, void 0, function* () {
                    yield api.exec('PUT', 'api/items/root:/testing:/content', {}, randomContent(), {
                        'Content-Type': 'application/octet-stream',
                    });
                });
                yield apiPut();
                const promises = [];
                for (let i = 0; i < 100; i++) {
                    promises.push(void apiPut());
                }
                yield Promise.all(promises);
                // eslint-disable-next-line no-console
                console.info(yield api.exec('GET', 'api/items/root:/testing:'));
            }
            yield Promise.all(promises);
        });
    }
}
module.exports = Command;
//# sourceMappingURL=command-testing.js.map