"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const vm = require("vm");
const BasePluginRunner_1 = require("@joplin/lib/services/plugins/BasePluginRunner");
const executeSandboxCall_1 = require("@joplin/lib/services/plugins/utils/executeSandboxCall");
const mapEventHandlersToIds_1 = require("@joplin/lib/services/plugins/utils/mapEventHandlersToIds");
const uuid_1 = require("@joplin/lib/uuid");
const sandboxProxy = require('@joplin/lib/services/plugins/sandboxProxy');
function createConsoleWrapper(pluginId) {
    const wrapper = {};
    for (const n in console) {
        // eslint-disable-next-line no-console
        if (!console.hasOwnProperty(n))
            continue;
        wrapper[n] = (...args) => {
            const newArgs = args.slice();
            newArgs.splice(0, 0, `Plugin "${pluginId}":`);
            return console[n](...newArgs);
        };
    }
    return wrapper;
}
// The CLI plugin runner is more complex than it needs to be because it more or less emulates
// how it would work in a multi-process architecture, as in the desktop app (and probably how
// it would work in the mobile app too). This is mainly to allow doing integration testing.
//
// For example, all plugin calls go through a proxy, however they could made directly since
// the plugin script is running within the same process as the main app.
class PluginRunner extends BasePluginRunner_1.default {
    constructor() {
        super();
        this.eventHandlers_ = {};
        this.activeSandboxCalls_ = {};
        this.eventHandler = this.eventHandler.bind(this);
    }
    eventHandler(eventHandlerId, args) {
        return __awaiter(this, void 0, void 0, function* () {
            const cb = this.eventHandlers_[eventHandlerId];
            return cb(...args);
        });
    }
    newSandboxProxy(pluginId, sandbox) {
        const target = (path, args) => __awaiter(this, void 0, void 0, function* () {
            const callId = `${pluginId}::${path}::${uuid_1.default.createNano()}`;
            this.activeSandboxCalls_[callId] = true;
            const promise = (0, executeSandboxCall_1.default)(pluginId, sandbox, `joplin.${path}`, (0, mapEventHandlersToIds_1.default)(args, this.eventHandlers_), this.eventHandler);
            // eslint-disable-next-line promise/prefer-await-to-then -- Old code before rule was applied
            void promise.finally(() => {
                delete this.activeSandboxCalls_[callId];
            });
            return promise;
        });
        return {
            joplin: sandboxProxy(target),
            console: createConsoleWrapper(pluginId),
        };
    }
    run(plugin, sandbox) {
        return __awaiter(this, void 0, void 0, function* () {
            // eslint-disable-next-line @typescript-eslint/ban-types -- Old code before rule was applied
            return new Promise((resolve, reject) => {
                const onStarted = () => {
                    plugin.off('started', onStarted);
                    resolve();
                };
                plugin.on('started', onStarted);
                const vmSandbox = vm.createContext(this.newSandboxProxy(plugin.id, sandbox));
                try {
                    vm.runInContext(plugin.scriptText, vmSandbox);
                }
                catch (error) {
                    reject(error);
                }
            });
        });
    }
    waitForSandboxCalls() {
        return __awaiter(this, void 0, void 0, function* () {
            const startTime = Date.now();
            // eslint-disable-next-line @typescript-eslint/ban-types -- Old code before rule was applied
            return new Promise((resolve, reject) => {
                const iid = setInterval(() => {
                    if (!Object.keys(this.activeSandboxCalls_).length) {
                        clearInterval(iid);
                        resolve();
                    }
                    if (Date.now() - startTime > 4000) {
                        clearInterval(iid);
                        reject(new Error(`Timeout while waiting for sandbox calls to complete: ${JSON.stringify(this.activeSandboxCalls_)}`));
                    }
                }, 10);
            });
        });
    }
}
exports.default = PluginRunner;
//# sourceMappingURL=PluginRunner.js.map